@@ -21,7 +21,10 @@ import java.io.File;
 import java.io.IOException;
 import java.lang.reflect.UndeclaredThrowableException;
 import java.nio.charset.Charset;
+import java.nio.file.Files;
 import java.util.ArrayList;
+import java.util.Comparator;
+import java.util.Date;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -36,6 +39,7 @@ import org.codehaus.plexus.archiver.manager.ArchiverManager;
 import org.codehaus.plexus.archiver.manager.NoSuchArchiverException;
 import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
 import org.codehaus.plexus.components.io.attributes.PlexusIoResourceAttributes;
+import org.codehaus.plexus.components.io.attributes.SimpleResourceAttributes;
 import org.codehaus.plexus.components.io.functions.ResourceAttributeSupplier;
 import org.codehaus.plexus.components.io.resources.AbstractPlexusIoResourceCollection;
 import org.codehaus.plexus.components.io.resources.EncodingSupported;
@@ -50,7 +54,6 @@ import org.codehaus.plexus.logging.AbstractLogEnabled;
 import org.codehaus.plexus.logging.Logger;
 import org.codehaus.plexus.logging.console.ConsoleLogger;
 import org.codehaus.plexus.personality.plexus.lifecycle.phase.Contextualizable;
-import org.codehaus.plexus.util.Os;
 import static org.codehaus.plexus.archiver.util.DefaultArchivedFileSet.archivedFileSet;
 import static org.codehaus.plexus.archiver.util.DefaultFileSet.fileSet;
 
@@ -72,7 +75,7 @@ public abstract class AbstractArchiver
      * of {@link ArchiveEntry} by {@link #getResources()}.
      * </ul>
      */
-    private final List<Object> resources = new ArrayList<Object>();
+    private final List<Object> resources = new ArrayList<>();
 
     private boolean includeEmptyDirs = true;
 
@@ -95,7 +98,7 @@ public abstract class AbstractArchiver
     // On lunix-like systems, we replace windows backslashes with forward slashes
     private final boolean replacePathSlashesToJavaPaths = File.separatorChar == '/';
 
-    private final List<Closeable> closeables = new ArrayList<Closeable>();
+    private final List<Closeable> closeables = new ArrayList<>();
 
     /**
      * since 2.2 is on by default
@@ -104,6 +107,36 @@ public abstract class AbstractArchiver
      */
     private boolean useJvmChmod = true;
 
+    /**
+     * @since 4.2.0
+     */
+    private Date lastModifiedDate;
+
+    /**
+     * @since 4.2.0
+     */
+    private Comparator<String> filenameComparator;
+
+    /**
+     * @since 4.2.0
+     */
+    private int overrideUid = -1;
+
+    /**
+     * @since 4.2.0
+     */
+    private String overrideUserName;
+
+    /**
+     * @since 4.2.0
+     */
+    private int overrideGid = -1;
+
+    /**
+     * @since 4.2.0
+     */
+    private String overrideGroupName;
+
     // contextualized.
     private ArchiverManager archiverManager;
 
@@ -346,10 +379,15 @@ public abstract class AbstractArchiver
         collection.setCaseSensitive( fileSet.isCaseSensitive() );
         collection.setUsingDefaultExcludes( fileSet.isUsingDefaultExcludes() );
         collection.setStreamTransformer( fileSet.getStreamTransformer() );
+        collection.setFileMappers( fileSet.getFileMappers() );
+        collection.setFilenameComparator( getFilenameComparator() );
 
-        if ( getOverrideDirectoryMode() > -1 || getOverrideFileMode() > -1 )
+        if ( getOverrideDirectoryMode() > -1 || getOverrideFileMode() > -1 || getOverrideUid() > -1
+            || getOverrideGid() > -1 || getOverrideUserName() != null || getOverrideGroupName() != null )
         {
-            collection.setOverrideAttributes( -1, null, -1, null, getOverrideFileMode(), getOverrideDirectoryMode() );
+            collection.setOverrideAttributes( getOverrideUid(), getOverrideUserName(), getOverrideGid(),
+                                              getOverrideGroupName(), getOverrideFileMode(),
+                                              getOverrideDirectoryMode() );
         }
 
         if ( getDefaultDirectoryMode() > -1 || getDefaultFileMode() > -1 )
@@ -386,6 +424,18 @@ public abstract class AbstractArchiver
             ArchiveEntry.createSymlinkEntry( symlinkName, permissions, symlinkDestination, getDirectoryMode() ) );
     }
 
+    private ArchiveEntry updateArchiveEntryAttributes( ArchiveEntry entry )
+    {
+        if ( getOverrideUid() > -1 || getOverrideGid() > -1 || getOverrideUserName() != null
+            || getOverrideGroupName() != null )
+        {
+            entry.setResourceAttributes( new SimpleResourceAttributes( getOverrideUid(), getOverrideUserName(),
+                                                                       getOverrideGid(), getOverrideGroupName(),
+                                                                       entry.getMode() ) );
+        }
+        return entry;
+    }
+
     protected ArchiveEntry asArchiveEntry( @Nonnull final PlexusIoResource resource, final String destFileName,
                                            final int permissions, PlexusIoResourceCollection collection )
         throws ArchiverException
@@ -395,14 +445,17 @@ public abstract class AbstractArchiver
             throw new ArchiverException( resource.getName() + " not found." );
         }
 
+        ArchiveEntry entry;
         if ( resource.isFile() )
         {
-            return ArchiveEntry.createFileEntry( destFileName, resource, permissions, collection, getDirectoryMode() );
+            entry = ArchiveEntry.createFileEntry( destFileName, resource, permissions, collection, getDirectoryMode() );
         }
         else
         {
-            return ArchiveEntry.createDirectoryEntry( destFileName, resource, permissions, getDirectoryMode() );
+            entry = ArchiveEntry.createDirectoryEntry( destFileName, resource, permissions, getDirectoryMode() );
         }
+
+        return updateArchiveEntryAttributes( entry );
     }
 
     private ArchiveEntry asArchiveEntry( final AddedResourceCollection collection, final PlexusIoResource resource )
@@ -455,7 +508,9 @@ public abstract class AbstractArchiver
         try
         {
             // do a null check here, to avoid creating a file stream if there are no filters...
-            doAddResource( ArchiveEntry.createFileEntry( destFileName, inputFile, permissions, getDirectoryMode() ) );
+            ArchiveEntry entry =
+                ArchiveEntry.createFileEntry( destFileName, inputFile, permissions, getDirectoryMode() );
+            doAddResource( updateArchiveEntryAttributes( entry ) );
         }
         catch ( final IOException e )
         {
@@ -640,7 +695,7 @@ public abstract class AbstractArchiver
     {
         try
         {
-            final Map<String, ArchiveEntry> map = new HashMap<String, ArchiveEntry>();
+            final Map<String, ArchiveEntry> map = new HashMap<>();
             for ( final ResourceIterator iter = getResources(); iter.hasNext(); )
             {
                 final ArchiveEntry entry = iter.next();
@@ -668,7 +723,7 @@ public abstract class AbstractArchiver
     {
         this.destFile = destFile;
 
-        if ( destFile != null )
+        if ( destFile != null && destFile.getParentFile() != null )
         {
             destFile.getParentFile().mkdirs();
         }
@@ -737,6 +792,7 @@ public abstract class AbstractArchiver
         proxy.setUsingDefaultExcludes( fileSet.isUsingDefaultExcludes() );
         proxy.setFileSelectors( fileSet.getFileSelectors() );
         proxy.setStreamTransformer( fileSet.getStreamTransformer() );
+        proxy.setFileMappers( fileSet.getFileMappers() );
 
         if ( getOverrideDirectoryMode() > -1 || getOverrideFileMode() > -1 )
         {
@@ -885,12 +941,12 @@ public abstract class AbstractArchiver
         throws ArchiverException
     {
         final File zipFile = getDestFile();
-        final long destTimestamp = zipFile.lastModified();
-        if ( destTimestamp == 0 )
+        if ( !zipFile.exists() )
         {
             getLogger().debug( "isUp2date: false (Destination " + zipFile.getPath() + " not found.)" );
             return false; // File doesn't yet exist
         }
+        final long destTimestamp = getFileLastModifiedTime(zipFile);
 
         final Iterator it = resources.iterator();
         if ( !it.hasNext() )
@@ -939,6 +995,26 @@ public abstract class AbstractArchiver
         return true;
     }
 
+    /**
+     * Returns the last modified time in milliseconds of a file.
+     * It avoids the bug where milliseconds precision is lost on File#lastModified (JDK-8177809) on JDK8 and Linux.
+     * @param file The file where the last modified time will be returned for.
+     * @return The last modified time in milliseconds of the file.
+     * @throws ArchiverException In the case of an IOException, for example when the file does not exists.
+     */
+    private long getFileLastModifiedTime( File file )
+            throws ArchiverException
+    {
+        try
+        {
+            return Files.getLastModifiedTime( file.toPath() ).toMillis();
+        }
+        catch ( IOException e )
+        {
+            throw new ArchiverException( e.getMessage(), e );
+        }
+    }
+
     protected boolean checkForced()
         throws ArchiverException
     {
@@ -1132,4 +1208,115 @@ public abstract class AbstractArchiver
         this.ignorePermissions = ignorePermissions;
     }
 
+    @Override
+    public void setLastModifiedDate( Date lastModifiedDate )
+    {
+        this.lastModifiedDate = lastModifiedDate;
+    }
+
+    @Override
+    public Date getLastModifiedDate()
+    {
+        return lastModifiedDate;
+    }
+
+    @Override
+    public void setFilenameComparator( Comparator<String> filenameComparator )
+    {
+        this.filenameComparator = filenameComparator;
+    }
+
+    public Comparator<String> getFilenameComparator()
+    {
+        return filenameComparator;
+    }
+
+    @Override
+    public void setOverrideUid( int uid )
+    {
+        overrideUid = uid;
+    }
+
+    @Override
+    public void setOverrideUserName( String userName )
+    {
+        overrideUserName = userName;
+    }
+
+    @Override
+    public int getOverrideUid()
+    {
+        return overrideUid;
+    }
+
+    @Override
+    public String getOverrideUserName()
+    {
+        return overrideUserName;
+    }
+
+    @Override
+    public void setOverrideGid( int gid )
+    {
+        overrideGid = gid;
+    }
+
+    @Override
+    public void setOverrideGroupName( String groupName )
+    {
+        overrideGroupName = groupName;
+    }
+
+    @Override
+    public int getOverrideGid()
+    {
+        return overrideGid;
+    }
+
+    @Override
+    public String getOverrideGroupName()
+    {
+        return overrideGroupName;
+    }
+
+    @Override
+    public void configureReproducible( Date lastModifiedDate )
+    {
+        // 1. force last modified date
+        setLastModifiedDate( normalizeLastModifiedDate( lastModifiedDate ) );
+
+        // 2. sort filenames in each directory when scanning filesystem
+        setFilenameComparator( new Comparator<String>()
+        {
+            @Override
+            public int compare( String s1, String s2 )
+            {
+                return s1.compareTo( s2 );
+            }
+        } );
+
+        // 3. ignore file/directory mode from filesystem, since they may vary based on local user umask
+        // notice: this overrides execute bit on Unix (that is already ignored on Windows)
+        setFileMode( Archiver.DEFAULT_FILE_MODE );
+        setDirectoryMode( Archiver.DEFAULT_DIR_MODE );
+
+        // 4. ignore uid/gid from filesystem (for tar)
+        setOverrideUid( 0 );
+        setOverrideUserName( "root" ); // is it possible to avoid this, like "tar --numeric-owner"?
+        setOverrideGid( 0 );
+        setOverrideGroupName( "root" );
+    }
+
+    /**
+     * Normalize last modified time value to get reproducible archive entries, based on
+     * archive binary format (tar uses UTC timestamp but zip uses local time then requires
+     * tweaks to make the value reproducible whatever the current timezone is).
+     *
+     * @param lastModifiedDate 
+     * @return
+     */
+    protected Date normalizeLastModifiedDate( Date lastModifiedDate )
+    {
+        return lastModifiedDate;
+    }
 }
