@@ -16,10 +16,13 @@
  */
 package org.codehaus.plexus.archiver.jar;
 
+import static org.codehaus.plexus.archiver.util.Streams.bufferedOutputStream;
+import static org.codehaus.plexus.archiver.util.Streams.fileInputStream;
+import static org.codehaus.plexus.archiver.util.Streams.fileOutputStream;
+
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStreamWriter;
@@ -35,17 +38,16 @@ import java.util.SortedMap;
 import java.util.StringTokenizer;
 import java.util.TreeMap;
 import java.util.Vector;
+
 import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
 import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
+import org.apache.commons.compress.archivers.zip.ZipFile;
 import org.apache.commons.compress.parallel.InputStreamSupplier;
 import org.codehaus.plexus.archiver.ArchiverException;
 import org.codehaus.plexus.archiver.zip.ConcurrentJarCreator;
 import org.codehaus.plexus.archiver.zip.ZipArchiver;
 import org.codehaus.plexus.logging.Logger;
 import org.codehaus.plexus.logging.console.ConsoleLogger;
-import org.codehaus.plexus.util.IOUtil;
-import static org.codehaus.plexus.archiver.util.Streams.bufferedOutputStream;
-import static org.codehaus.plexus.archiver.util.Streams.fileOutputStream;
 
 /**
  * Base class for tasks that build archives in JAR file format.
@@ -144,6 +146,11 @@ public class JarArchiver
      */
     private ArrayList<String> indexJars;
 
+    /**
+     * Creates a minimal default manifest with {@code Manifest-Version: 1.0} only.
+     */
+    private boolean minimalDefaultManifest = false;
+
     /**
      * constructor
      */
@@ -166,6 +173,15 @@ public class JarArchiver
         index = flag;
     }
 
+    /**
+     * Set whether the default manifest is minimal, thus having only {@code Manifest-Version: 1.0} in it.
+     *
+     * @param minimalDefaultManifest true to create minimal default manifest
+     */
+    public void setMinimalDefaultManifest( boolean minimalDefaultManifest ) {
+        this.minimalDefaultManifest = minimalDefaultManifest;
+    }
+
     @SuppressWarnings(
     {
         "JavaDoc", "UnusedDeclaration"
@@ -181,7 +197,7 @@ public class JarArchiver
      *
      * @param newManifest The new manifest
      *
-     * @throws ManifestException .
+     * @throws ManifestException
      */
     public void addConfiguredManifest( Manifest newManifest )
         throws ManifestException
@@ -204,7 +220,6 @@ public class JarArchiver
      * @param manifestFile the manifest file to use.
      *
      * @throws org.codehaus.plexus.archiver.ArchiverException
-     * .
      */
     @SuppressWarnings(
     {
@@ -224,24 +239,15 @@ public class JarArchiver
     private Manifest getManifest( File manifestFile )
         throws ArchiverException
     {
-        InputStream in = null;
-        try
+        try ( InputStream in = fileInputStream( manifestFile ) )
         {
-            in = new FileInputStream( manifestFile );
-            final Manifest mf = getManifest( in );
-            in.close();
-            in = null;
-            return mf;
+            return getManifest( in );
         }
         catch ( IOException e )
         {
             throw new ArchiverException( "Unable to read manifest file: " + manifestFile + " (" + e.getMessage() + ")",
                                          e );
         }
-        finally
-        {
-            IOUtil.close( in );
-        }
     }
 
     private Manifest getManifest( InputStream is )
@@ -264,9 +270,9 @@ public class JarArchiver
      * other specified manifests.
      * "mergewithoutmain" merges everything but the Main section of the manifests.
      * Default value is "skip".
-     * <p/>
+     * <p>
      * Note: if this attribute's value is not "skip", the created jar will not
-     * be readable by using java.util.jar.JarInputStream
+     * be readable by using java.util.jar.JarInputStream</p>
      *
      * @param config setting for found manifest behavior.
      */
@@ -319,10 +325,19 @@ public class JarArchiver
                    || super.hasVirtualFiles();
     }
 
-    private Manifest createManifest()
+    /**
+     * Creates the manifest to be added to the JAR archive.
+     * Sub-classes may choose to override this method
+     * in order to inspect or modify the JAR manifest file.
+     *
+     * @return the manifest for the JAR archive.
+     *
+     * @throws ArchiverException
+     */
+    protected Manifest createManifest()
         throws ArchiverException
     {
-        Manifest finalManifest = Manifest.getDefaultManifest();
+        Manifest finalManifest = Manifest.getDefaultManifest( minimalDefaultManifest );
 
         if ( ( manifest == null ) && ( manifestFile != null ) )
         {
@@ -352,7 +367,7 @@ public class JarArchiver
     private void writeManifest( ConcurrentJarCreator zOut, Manifest manifest )
         throws IOException, ArchiverException
     {
-        for ( Enumeration e = manifest.getWarnings(); e.hasMoreElements(); )
+        for ( Enumeration<String> e = manifest.getWarnings(); e.hasMoreElements(); )
         {
             getLogger().warn( "Manifest warning: " + e.nextElement() );
         }
@@ -389,7 +404,6 @@ public class JarArchiver
      * @throws IOException thrown if there is an error while creating the
      * index and adding it to the zip stream.
      * @throws org.codehaus.plexus.archiver.ArchiverException
-     * .
      */
     private void createIndexList( ConcurrentJarCreator zOut )
         throws IOException, ArchiverException
@@ -683,18 +697,15 @@ public class JarArchiver
 
     /**
      * try to guess the name of the given file.
-     * <p/>
      * <p>
      * If this jar has a classpath attribute in its manifest, we
      * can assume that it will only require an index of jars listed
      * there. try to find which classpath entry is most likely the
      * one the given file name points to.</p>
-     * <p/>
      * <p>
      * In the absence of a classpath attribute, assume the other
      * files will be placed inside the same directory as this jar and
      * use their basename.</p>
-     * <p/>
      * <p>
      * if there is a classpath and the given file doesn't match any
      * of its entries, return null.</p>
@@ -761,7 +772,7 @@ public class JarArchiver
      * @param files .
      * @param dirs .
      *
-     * @throws java.io.IOException .
+     * @throws java.io.IOException
      */
     protected static void grabFilesAndDirs( String file, List<String> dirs, List<String> files )
         throws IOException
@@ -779,10 +790,8 @@ public class JarArchiver
         }
         else
         {
-            org.apache.commons.compress.archivers.zip.ZipFile zf = null;
-            try
+            try ( ZipFile zf = new ZipFile( file, "utf-8" ) )
             {
-                zf = new org.apache.commons.compress.archivers.zip.ZipFile( file, "utf-8" );
                 Enumeration<ZipArchiveEntry> entries = zf.getEntries();
                 HashSet<String> dirSet = new HashSet<String>();
                 while ( entries.hasMoreElements() )
@@ -813,13 +822,6 @@ public class JarArchiver
                 }
                 dirs.addAll( dirSet );
             }
-            finally
-            {
-                if ( zf != null )
-                {
-                    zf.close();
-                }
-            }
         }
     }
 
