@@ -16,12 +16,15 @@
  */
 package org.codehaus.plexus.archiver.tar;
 
+import static org.codehaus.plexus.archiver.util.Streams.bufferedOutputStream;
+
 import java.io.File;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.nio.file.Files;
 import java.util.zip.GZIPOutputStream;
+
 import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
 import org.apache.commons.compress.archivers.tar.TarArchiveOutputStream;
 import org.apache.commons.compress.compressors.bzip2.BZip2CompressorOutputStream;
@@ -39,7 +42,6 @@ import org.codehaus.plexus.components.io.resources.PlexusIoResource;
 import org.codehaus.plexus.util.IOUtil;
 import org.codehaus.plexus.util.StringUtils;
 import org.iq80.snappy.SnappyOutputStream;
-import static org.codehaus.plexus.archiver.util.Streams.bufferedOutputStream;
 
 /**
  * @author <a href="mailto:evenisse@codehaus.org">Emmanuel Venisse</a>
@@ -64,16 +66,16 @@ public class TarArchiver
     /**
      * Set how to handle long files, those with a path&gt;100 chars.
      * Optional, default=warn.
-     * <p/>
-     * Allowable values are
+     * <p>
+     * Allowable values are </p>
      * <ul>
-     * <li> truncate - paths are truncated to the maximum length
-     * <li> fail - paths greater than the maximum cause a build exception
-     * <li> warn - paths greater than the maximum cause a warning and GNU is used
-     * <li> gnu - GNU extensions are used for any paths greater than the maximum.
-     * <li> posix - posix extensions are used for any paths greater than the maximum.
-     * <li> posixwarn - posix extensions are used (with warning) for any paths greater than the maximum.
-     * <li> omit - paths greater than the maximum are omitted from the archive
+     * <li> truncate - paths are truncated to the maximum length </li>
+     * <li> fail - paths greater than the maximum cause a build exception </li>
+     * <li> warn - paths greater than the maximum cause a warning and GNU is used </li>
+     * <li> gnu - GNU extensions are used for any paths greater than the maximum. </li>
+     * <li> posix - posix extensions are used for any paths greater than the maximum. </li>
+     * <li> posixwarn - posix extensions are used (with warning) for any paths greater than the maximum. </li>
+     * <li> omit - paths greater than the maximum are omitted from the archive </li>
      * </ul>
      *
      * @param mode the mode to handle long file names.
@@ -133,7 +135,7 @@ public class TarArchiver
 
         try
         {
-            tOut = new TarArchiveOutputStream( compress( compression, new FileOutputStream( tarFile ) ), "UTF8" );
+            tOut = new TarArchiveOutputStream( compress( compression, Files.newOutputStream( tarFile.toPath() ) ), "UTF8" );
             if ( longFileMode.isTruncateMode() )
             {
                 tOut.setLongFileMode( TarArchiveOutputStream.LONGFILE_TRUNCATE );
@@ -281,10 +283,17 @@ public class TarArchiver
                 te = new TarArchiveEntry( vPath );
             }
 
-            long teLastModified = entry.getResource().getLastModified();
-            te.setModTime( teLastModified == PlexusIoResource.UNKNOWN_MODIFICATION_DATE
-                               ? System.currentTimeMillis()
-                               : teLastModified );
+            if ( getLastModifiedDate() == null )
+            {
+                long teLastModified = entry.getResource().getLastModified();
+                te.setModTime( teLastModified == PlexusIoResource.UNKNOWN_MODIFICATION_DATE
+                                   ? System.currentTimeMillis()
+                                   : teLastModified );
+            }
+            else
+            {
+                te.setModTime( getLastModifiedDate() );
+            }
 
             if ( entry.getType() == ArchiveEntry.SYMLINK )
             {
@@ -481,7 +490,7 @@ public class TarArchiver
     {
         if ( TarCompressionMethod.gzip.equals( tarCompressionMethod ) )
         {
-            return Streams.bufferedOutputStream( new GZIPOutputStream( ostream ) );
+            return bufferedOutputStream( new GZIPOutputStream( ostream ) );
         }
         else if ( TarCompressionMethod.bzip2.equals( tarCompressionMethod ) )
         {
