@@ -23,14 +23,20 @@
  */
 package org.codehaus.plexus.archiver.tar;
 
+import static org.codehaus.plexus.archiver.util.Streams.bufferedInputStream;
+import static org.codehaus.plexus.components.io.resources.ResourceFactory.createResource;
+
 import java.io.BufferedInputStream;
 import java.io.File;
 import java.io.FileInputStream;
-import java.io.FileWriter;
 import java.io.IOException;
+import java.io.Writer;
+import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
 import java.util.Enumeration;
 import java.util.LinkedHashMap;
 import java.util.Map;
+
 import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
 import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
 import org.codehaus.plexus.PlexusTestCase;
@@ -50,8 +56,6 @@ import org.codehaus.plexus.util.FileUtils;
 import org.codehaus.plexus.util.IOUtil;
 import org.codehaus.plexus.util.Os;
 import org.junit.Assert;
-import static org.codehaus.plexus.archiver.util.Streams.bufferedInputStream;
-import static org.codehaus.plexus.components.io.resources.ResourceFactory.createResource;
 
 /**
  * @author Emmanuel Venisse
@@ -260,23 +264,15 @@ public class TarArchiverTest
         throws IOException, ArchiverException
     {
         File file = new File( dir, fname );
-        FileWriter writer = null;
 
-        try
+        if ( file.getParentFile() != null )
         {
-            if ( file.getParentFile() != null )
-            {
-                file.getParentFile().mkdirs();
-            }
-
-            writer = new FileWriter( file );
-            writer.write( "This is a test file." );
-            writer.close();
-            writer = null;
+            file.getParentFile().mkdirs();
         }
-        finally
+
+        try ( Writer writer = Files.newBufferedWriter( file.toPath(), StandardCharsets.UTF_8 ) )
         {
-            IOUtil.close( writer );
+            writer.write( "This is a test file." );
         }
 
         ArchiveEntryUtils.chmod( file, mode );
@@ -323,7 +319,7 @@ public class TarArchiverTest
 
         TarArchiveInputStream tis;
 
-        tis = new TarArchiveInputStream( bufferedInputStream( new FileInputStream( archiver.getDestFile() ) ) );
+        tis = new TarArchiveInputStream( bufferedInputStream( Files.newInputStream( archiver.getDestFile().toPath() ) ) );
         TarArchiveEntry te;
 
         while ( ( te = tis.getNextTarEntry() ) != null )
@@ -380,7 +376,7 @@ public class TarArchiverTest
 
         TarArchiveInputStream tis;
 
-        tis = new TarArchiveInputStream( new BufferedInputStream( new FileInputStream( archiver.getDestFile() ) ) );
+        tis = new TarArchiveInputStream( new BufferedInputStream( Files.newInputStream( archiver.getDestFile().toPath() ) ) );
         TarArchiveEntry te;
 
         while ( ( te = tis.getNextTarEntry() ) != null )
