@@ -30,14 +30,24 @@ import java.io.FileInputStream;
 import java.io.FileWriter;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.Writer;
+import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
+import java.nio.file.attribute.FileTime;
+import java.text.DateFormat;
+import java.text.ParseException;
+import java.text.SimpleDateFormat;
 import java.util.Arrays;
 import java.util.Date;
 import java.util.Enumeration;
 import java.util.Map;
+import java.util.TimeZone;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipInputStream;
 import java.util.zip.ZipOutputStream;
+
 import javax.annotation.Nonnull;
+
 import org.apache.commons.compress.archivers.zip.ExtraFieldUtils;
 import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
 import org.apache.commons.compress.archivers.zip.ZipExtraField;
@@ -59,6 +69,8 @@ import org.codehaus.plexus.components.io.attributes.FileAttributes;
 import org.codehaus.plexus.components.io.attributes.PlexusIoResourceAttributeUtils;
 import org.codehaus.plexus.components.io.attributes.PlexusIoResourceAttributes;
 import org.codehaus.plexus.components.io.attributes.SimpleResourceAttributes;
+import org.codehaus.plexus.components.io.filemappers.FileMapper;
+import org.codehaus.plexus.components.io.filemappers.PrefixFileMapper;
 import org.codehaus.plexus.components.io.functions.InputStreamTransformer;
 import org.codehaus.plexus.components.io.resources.PlexusIoFileResourceCollection;
 import org.codehaus.plexus.components.io.resources.PlexusIoResource;
@@ -332,23 +344,15 @@ public class ZipArchiverTest
         throws IOException, ArchiverException
     {
         File file = new File( dir, fname );
-        FileWriter writer = null;
 
-        try
+        if ( file.getParentFile() != null )
         {
-            if ( file.getParentFile() != null )
-            {
-                file.getParentFile().mkdirs();
-            }
-
-            writer = new FileWriter( file );
-            writer.write( "This is a test file." );
-            writer.close();
-            writer = null;
+            file.getParentFile().mkdirs();
         }
-        finally
+
+        try ( Writer writer = Files.newBufferedWriter( file.toPath(), StandardCharsets.UTF_8 ) )
         {
-            IOUtil.close( writer );
+            writer.write( "This is a test file." );
         }
 
         ArchiveEntryUtils.chmod( file, mode );
@@ -399,13 +403,13 @@ public class ZipArchiverTest
         assertEquals( ZipEntry.STORED, rarEntry.getMethod() );
         assertEquals( ZipEntry.DEFLATED, tarEntry.getMethod() );
         // ...and no file is corrupted in the process
-        assertTrue( IOUtil.contentEquals( new FileInputStream( getTestFile( "src/test/jars/test.zip" ) ),
+        assertTrue( IOUtil.contentEquals( Files.newInputStream( getTestFile( "src/test/jars/test.zip" ).toPath() ),
                     zfDontRecompress.getInputStream( zipEntry ) ) );
-        assertTrue( IOUtil.contentEquals( new FileInputStream( getTestFile( "src/test/jars/test.jar" ) ),
+        assertTrue( IOUtil.contentEquals( Files.newInputStream( getTestFile( "src/test/jars/test.jar" ).toPath() ),
                 zfDontRecompress.getInputStream( jarEntry ) ) );
-        assertTrue( IOUtil.contentEquals( new FileInputStream( getTestFile( "src/test/jars/test.rar" ) ),
+        assertTrue( IOUtil.contentEquals( Files.newInputStream( getTestFile( "src/test/jars/test.rar" ).toPath() ),
                 zfDontRecompress.getInputStream( rarEntry ) ) );
-        assertTrue( IOUtil.contentEquals( new FileInputStream( getTestFile( "src/test/jars/test.tar.gz" ) ),
+        assertTrue( IOUtil.contentEquals( Files.newInputStream( getTestFile( "src/test/jars/test.tar.gz" ).toPath() ),
                 zfDontRecompress.getInputStream( tarEntry ) ) );
         zfDontRecompress.close();
     }
@@ -429,6 +433,9 @@ public class ZipArchiverTest
 
         };
         sfd.setStreamTransformer( is );
+        PrefixFileMapper mapper = new PrefixFileMapper();
+        mapper.setPrefix( "prefix" );
+        sfd.setFileMappers( new FileMapper[] { mapper } );
         zipArchiver.addArchivedFileSet( sfd );
         zipArchiver.createArchive();
 
@@ -438,7 +445,7 @@ public class ZipArchiverTest
         zipUnArchiver.setDestFile( destFile );
         zipUnArchiver.extract();
         File a3byteFile = new File( destFile,
-                                    "Users/kristian/lsrc/plexus/plexus-archiver/src/main/java/org/codehaus/plexus/archiver/zip/ZipArchiver.java" );
+                                    "prefixUsers/kristian/lsrc/plexus/plexus-archiver/src/main/java/org/codehaus/plexus/archiver/zip/ZipArchiver.java" );
         assertTrue( a3byteFile.exists() );
         assertTrue( a3byteFile.length() == 3 );
     }
@@ -622,6 +629,33 @@ public class ZipArchiverTest
         ArchiveFileComparator.assertEquals( cmp1, cmp2, "" );
     }
 
+    /*
+     * Zip archives store file modification times with a granularity of two seconds.
+     * Verify that ZipArchiver rounds up the last modified time.
+     */
+    public void testLastModifiedTimeRounding()
+        throws Exception
+    {
+        File oddSecondsTimestampFile = File.createTempFile( "odd-seconds-timestamp", null );
+        oddSecondsTimestampFile.deleteOnExit();
+        // The milliseconds part is set to zero as not all filesystem support timestamp more granular than second.
+        Files.setLastModifiedTime( oddSecondsTimestampFile.toPath(), FileTime.fromMillis( 1534189011_000L ) );
+        File evenSecondsTimestampFile = File.createTempFile( "even-seconds-timestamp", null );
+        evenSecondsTimestampFile.deleteOnExit();
+        Files.setLastModifiedTime( evenSecondsTimestampFile.toPath(), FileTime.fromMillis( 1534189012_000L ) );
+
+        File destFile = getTestFile( "target/output/last-modified-time.zip" );
+        ZipArchiver archiver = getZipArchiver( destFile );
+        archiver.addFile( oddSecondsTimestampFile, "odd-seconds" );
+        archiver.addFile( evenSecondsTimestampFile, "even-seconds" );
+        archiver.createArchive();
+
+        // verify that the last modified time of the entry is equal or newer than the original file
+        ZipFile resultingZipFile = new ZipFile( destFile );
+        assertEquals( 1534189012_000L, resultingZipFile.getEntry( "odd-seconds" ).getTime() );
+        assertEquals( 1534189012_000L, resultingZipFile.getEntry( "even-seconds" ).getTime() );
+    }
+
     /*
      */
     public void testForced()
@@ -860,4 +894,57 @@ public class ZipArchiverTest
         }
     }
 
+    public void testFixedEntryModificationTime()
+            throws IOException
+    {
+        final long almostMinDosTime = toLocalTimeZone( 315532802000L );
+
+        final File zipFile = getTestFile( "target/output/zip-with-fixed-entry-modification-times.zip" );
+        final ZipArchiver archiver = getZipArchiver( zipFile );
+        archiver.setLastModifiedDate( new Date( almostMinDosTime ) );
+        archiver.addDirectory( new File( "src/test/resources/zip-timestamp" ) );
+        archiver.createArchive();
+
+        assertTrue( zipFile.exists() );
+        try ( final ZipFile zf = new ZipFile( zipFile ) )
+        {
+            assertEquals( almostMinDosTime, zf.getEntry( "file-with-even-time.txt" ).getTime() );
+            assertEquals( almostMinDosTime, zf.getEntry( "file-with-odd-time.txt" ).getTime() );
+            assertEquals( almostMinDosTime, zf.getEntry( "foo/" ).getTime() );
+        }
+    }
+
+    /**
+     * Takes a timestamp, turns it into a textual representation based on GMT, then translated it into a timestamp in
+     * local timezone. This makes the test independent of the current TimeZone. The reason this is necessary is:
+     * <ul>
+     * <li>ZIP file format does not take timezone into account.</li>
+     * <li>In the process of converting the ZipEntry time from the DOS date format specified by the ZIP file format, the
+     * timestamp is converted to a Java Date object, which DOES depends of the current system TimeZone, therefore
+     * changing the value of the Date object representing that timestamp relative to the local TimeZone.</li>
+     * </ul>
+     *
+     * @param timestamp the epoch time to convert.
+     * @return the timestamp matching the same input date but in the local TZ.
+     */
+    private long toLocalTimeZone( long timestamp )
+    {
+        String dateFormat = "dd-MM-yyyy hh:mm:ss a";
+        DateFormat formatterWithTimeZone = new SimpleDateFormat( dateFormat );
+        formatterWithTimeZone.setTimeZone( TimeZone.getTimeZone( "GMT" ) );
+        String sDate = formatterWithTimeZone.format( new Date( timestamp ) );
+
+        DateFormat formatter = new SimpleDateFormat( dateFormat );
+        try
+        {
+            Date dateWithTimeZone = formatter.parse( sDate );
+            return dateWithTimeZone.getTime();
+        }
+        catch ( ParseException e )
+        {
+            fail( "Date '" + sDate + "' can not be parsed!" );
+            return 0L;
+        }
+    }
+
 }
